import unittest


class VDBTest(unittest.TestCase):
    pass

    # def test_evaluation_F23(self):
    #     excel_value = self.evaluator.get_cell_value('Sheet1!F23')
    #     value = self.evaluator.evaluate('Sheet1!F23')
    #     self.assertEqual( excel_value, value )
    #

    # def test_evaluation_F24(self):
    #     excel_value = self.evaluator.get_cell_value('Sheet1!F24')
    #     value = self.evaluator.evaluate('Sheet1!F24')
    #     self.assertEqual( excel_value, value )
    #
    #
    # def test_evaluation_F27(self):
    #     excel_value = self.evaluator.get_cell_value('Sheet1!F27')
    #     value = self.evaluator.evaluate('Sheet1!F27')
    #     self.assertEqual( excel_value, value )
    #
    #
    # def test_evaluation_F28(self):
    #     excel_value = self.evaluator.get_cell_value('Sheet1!F28')
    #     value = self.evaluator.evaluate('Sheet1!F28')
    #     self.assertEqual( excel_value, value )
